#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <ogcsys.h>
#include <gccore.h>
#include <wiiuse/wpad.h>
#include <fat.h>
#include <gcmodplay.h>

static u32 *xfb;
static GXRModeObj *rmode;

// Modplay
static MODPlay play; 
long mod_size;
char * buffer;
size_t result;

void Initialise() {
	
	VIDEO_Init();
	WPAD_Init();
	
	rmode = VIDEO_GetPreferredMode(NULL);
	
	xfb = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	console_init(xfb,20,20,rmode->fbWidth,rmode->xfbHeight,rmode->fbWidth*VI_DISPLAY_PIX_SZ);
	
	VIDEO_Configure(rmode);
	VIDEO_SetNextFramebuffer(xfb);
	VIDEO_SetBlack(FALSE);
	VIDEO_Flush();
	VIDEO_WaitVSync();
	if(rmode->viTVMode&VI_NON_INTERLACE) VIDEO_WaitVSync();
	
	AUDIO_Init(NULL);
	MODPlay_Init(&play);
}

int main() {
	
	Initialise();
	
	printf("\x1b[2;0H");
	
	if (!fatInitDefault()) {
		printf("Unable to initialise FAT subsystem, exiting");
		exit(0);
	}
	
	printf("Loading mod file.\n");
	
	FILE *f = fopen("/loop.mod", "rb");
	
	if (f == NULL) {
		fclose(f);
	}
	else {
		fseek (f , 0, SEEK_END);
		mod_size = ftell (f);
		rewind(f);
		
		// allocate memory to contain the whole file:
		buffer = (char*) malloc (sizeof(char)*mod_size);
		if (buffer == NULL) { perror ("Memory error\n"); }
		
		// copy the file into the buffer:
		result = fread (buffer,1,mod_size,f);
		if (result != mod_size) { perror ("Reading error\n"); }
		
		fclose(f);
		
		MODPlay_SetMOD(&play, buffer);
		MODPlay_Start(&play);
	}
	
	printf("Playing mod file. Press Home to exit\n");
	
	while(1) {
		
		WPAD_ScanPads();
		u32 pressed = WPAD_ButtonsDown(0);
		
		if (pressed & WPAD_BUTTON_HOME) {
			MODPlay_Stop(&play);
			exit(0);
		}
		
		VIDEO_WaitVSync();
	}
	
	return 0;
}
